@@ -27,17 +27,19 @@ use Crypt::KeyDerivation qw(hkdf);
 use Crypt::Mode::CFB;
 use Crypt::Mode::CTR;
 use Crypt::Mode::OFB;
+use Crypt::OpenSSL::Random qw(random_bytes);
 #use Crypt::NaCl::Sodium;
 use Crypt::Rabbit;
-use Crypt::Random qw( makerandom_octet );
-#use Crypt::RC4::XS;
+#use Crypt::Random qw( makerandom_octet );
+use Crypt::RC4::XS;
 use Crypt::Spritz;
 use Digest::MD5;
-use Digest::SHA;
+use Net::SSLeay;
+#use Digest::SHA;
 #use Mcrypt qw(:FUNCS);
 #use String::HexConvert ':all';
 
-our $VERSION = '0.9.1';
+our $VERSION = '0.9.2';
 
 our %_ciphers = 
     (
@@ -74,8 +76,7 @@ our %_ciphers =
         "chacha20-ietf" => [undef,undef,32,12],
         "chacha20-ietf-poly1305" =>  [undef,undef,32,12],
         "rabbit" => ['rabbit','stream',16,16],        
-        #"rc4-md5"  => ['arcfour','stream',16,16],
-        #"rc4-sha" => ['arcfour','stream',16,20],
+        "rc4-md5"  => ['arcfour','stream',16,16],
         "rc6-128-cfb" => ['RC6','cfb',16,16],
         "rc6-128-ctr" => ['RC6','ctr',16,16],
         "rc6-128-ofb" => ['RC6','ofb',16,16],
@@ -140,29 +141,20 @@ sub _initialize_cipher($$)
     my $_iv;
     my $_encrypt_nonce;
     my $_decrypt_nonce;
-    if (!defined($_method))
-    {
-        AE::log info => "Encryption method undefinde, using spritz." ;
 
-        $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
-        $_iv = makerandom_octet( Length => 16 );
-        $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
-    }
-    else
+    if($_method =~ /^chacha20/)
     {
-        if($_method =~ /^chacha20/)
-        {
-            if ($_method eq 'chacha20-ietf')
-            {
+        if ($_method eq 'chacha20-ietf')
+         {
                 $_encryptor = Crypt::NaCl::Sodium->stream();
                 #$_key = $_encryptor->chacha20_keygen();
                 $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
                 $_encrypt_nonce = $_encryptor->chacha20_ietf_nonce();
                 $_decryptor = Crypt::NaCl::Sodium->stream();
                 $_iv = $_encrypt_nonce;
-            }
-            else
-            {
+        }
+        else
+        {
                 $_encryptor = Crypt::NaCl::Sodium->aead();
                 #$_key = $_encryptor->chacha20_keygen();
                 $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
@@ -175,10 +167,10 @@ sub _initialize_cipher($$)
                 $_decrypt_nonce = $_decryptor->ietf_nonce("\0");
                 #carp $_encrypt_nonce;
                 #carp ascii_to_hex($_encrypt_nonce);
-            }
         }
-        elsif($_method =~ /gcm$/ )
-        {
+    }
+    elsif($_method =~ /gcm$/ )
+     {
             $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
             my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
             my $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
@@ -186,35 +178,32 @@ sub _initialize_cipher($$)
             $_encryptor = Crypt::AuthEnc::GCM->new('AES', $_encrypt_subkey);
             $_encrypt_nonce = Data::BytesLocker->new("\0" x 12);
             $_decrypt_nonce = Data::BytesLocker->new("\0" x 12);
-        }
-=cut for comment           
-            elsif($_method =~ /^rc4/ )
+        }           
+        elsif($_method =~ /^rc4/ )
+        {
+            $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+#                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+            $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
+            my $encrypt_rc4_key;
+            if ($_method eq 'rc4-md5' )
             {
-                $_encryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0],'',$_ciphers{$_method}->[1] , '' );
-                $_decryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0] ,'',$_ciphers{$_method}->[1], '' );
-                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
-
-                    my $encrypt_rc4_key;
-                    if ($_method eq 'rc4-md5' )
-                    {
-                       my $md = Digest::MD5->new();
-                       $md->add($_key . $_iv);
-                       $encrypt_rc4_key = $md->digest();
-                    }
-                    elsif($_method eq 'rc4-sha' )
-                    {
+                my $md = Digest::MD5->new();
+                $md->add($_key . $_iv);
+                $encrypt_rc4_key = $md->digest();
+            }
+            elsif($_method eq 'rc4-sha' )
+                {
                        my $sha = Digest::SHA->new();
                        $sha->add($_key . $_iv);
                        $encrypt_rc4_key = substr($sha->digest(),0,16);
-                    }
-                    Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
-            }
-=cut            
+                }
+                $_encryptor = Crypt::RC4::XS->new($encrypt_rc4_key);
+        }       
         elsif($_method eq 'rabbit')
         {
             $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-            $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+#            $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+            $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
             my $md = Digest::MD5->new();
             $md->add($_key . $_iv);
             my $encrypt_rabbit_key = $md->digest();
@@ -224,7 +213,8 @@ sub _initialize_cipher($$)
         elsif($_method eq "spritz") 
         {
             $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
-            $_iv = makerandom_octet( Length => 16 );
+#            $_iv = makerandom_octet( Length => 16 );
+            $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
             $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
         }
         elsif( ($_method =~ /^aes/  or $_method =~ /^camellia/ or $_method =~ /^rc6/ ) and $_method !~ /gcm$/ )
@@ -232,26 +222,38 @@ sub _initialize_cipher($$)
             if($_method =~ /cfb$/)
             {
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+#                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+                $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
                 $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
                 $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
             }
              elsif($_method =~ /ctr$/)
             {
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+#                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
                 $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
                 $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
             }
             elsif($_method =~ /ofb$/)
             {
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+#                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
                 $_encryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
                 $_decryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
             }
         }
+        else
+    {
+        AE::log info => "Encryption method undefined or wrong cipher." ;
+
+        $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
+#        $_iv = makerandom_octet( Length => 16 );
+        $_iv = Crypt::OpenSSL::Random::random_bytes($_ciphers{$_method}->[3]);
+        $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
     }
+    
     return $_encryptor, $_decryptor, $_key, $_encrypt_subkey,$_iv, $_encrypt_nonce,$_decrypt_nonce;
 }
 
@@ -358,7 +360,7 @@ Net::Shadowsocks - the asynchronous, non-blocking shadowsocks client and server.
 
 =head1 VERSION
 
-Version 0.9.1
+Version 0.9.2
 
 =head1 SYNOPSIS
     
@@ -366,7 +368,7 @@ Version 0.9.1
 
 Shadowsocks is a secure transport protocol based on SOCKS Protocol Version 5 (RFC 1928 ).Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭) protocol client and server. ssclient.pl is the asynchronous, non-blocking shadowsocks client. ssserver.pl is the asynchronous, non-blocking shadowsocks server. Run ssclient.pl and/or ssserver.pl and follow instructions.
 
-1. A total of 34 encryption methods are supported:
+1. A total of 35 encryption methods are supported:
 
 	aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
 	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
@@ -376,6 +378,7 @@ Shadowsocks is a secure transport protocol based on SOCKS Protocol Version 5 (RF
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
 	rabbit
+	rc4-md5
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
