@@ -32,7 +32,7 @@ package Net::Shadowsocks::Client;
     use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
     #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.1';
+    our $VERSION = '0.9.2';
     $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
@@ -51,6 +51,10 @@ package Net::Shadowsocks::Client;
             undef $self->{local_address};
             }
         }
+        unless (defined($self->{method}))
+        {
+            $self->{method} = 'spritz';
+        } 
 
         my $tcp_server;
         $tcp_server = AnyEvent::Socket::tcp_server(
@@ -60,7 +64,7 @@ package Net::Shadowsocks::Client;
             {
                 my ( $client_socket, $client_host, $client_port ) = @_;
 
-                AE::log info => "Got new client connection: $client_host:$client_port";
+                #AE::log info => "Got new client connection: $client_host:$client_port";
                 my $addr_to_send = '';
                 my $server_iv;
                 my $decrypt_subkey;
@@ -216,25 +220,17 @@ package Net::Shadowsocks::Client;
                                                         $server_iv = substr( $incoming_data, 0,length($iv) );
                                                         $incoming_data = substr($incoming_data,length($iv));
                                                         $rhandle->{rbuf} = $incoming_data;
-=cut for comment                                                        
-                                                        if ( !defined($self->{method}) or $self->{method} eq'rc4-md5' )
+                                                        
+                                                        if ( $self->{method} eq'rc4-md5' )
                                                         {
                                                             my $md = Digest::MD5->new();
                                                             $md->add($key . $server_iv);
                                                             #$md->add($server_iv);
                                                             my $decrypt_rc4_key = $md->digest();
-                                                            Mcrypt::mcrypt_init($decryptor,$decrypt_rc4_key,'' );
+                                                            $decryptor = Crypt::RC4::XS->new($decrypt_rc4_key);
                                                         }
-                                                        elsif($self->{method} eq'rc4-sha')
-                                                        {
-                                                            my $sha = Digest::SHA->new();
-                                                            $sha->add($key . $server_iv);
-                                                            #$md->add($server_iv);
-                                                            my $decrypt_rc4_key = substr($sha->digest(),0,16);
-                                                            Mcrypt::mcrypt_init($decryptor,$decrypt_rc4_key,'' );
-                                                        }
-=cut                                                     
-                                                        if ( !defined($self->{method}) or $self->{method} eq'spritz' )
+                                                        
+                                                        elsif ( !defined($self->{method}) or $self->{method} eq'spritz' )
                                                         {
                                                            
                                                             $decryptor = Crypt::Spritz::Cipher->new($key,$server_iv);
@@ -264,20 +260,20 @@ package Net::Shadowsocks::Client;
                                                 }
                                                 else
                                                 {
-=cut for comment
-                                                if (  $self->{method} =~ /^rc4/)
-                                                {
-                                                    $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
-                                                }
-=cut
-                                                    if (  $self->{method} eq "rabbit" or $self->{method} eq "spritz")
+
+                                                    if (  $self->{method} eq 'rc4-md5')
+                                                    {
+                                                        $decrypted_data = $decryptor->RC4($incoming_data );
+                                                    }
+
+                                                    elsif (  $self->{method} eq "rabbit" or $self->{method} eq "spritz")
                                                     {
                                                         $decrypted_data = $decryptor->decrypt($incoming_data);
                                                     }
-                                                elsif($self->{method} =~ /^chacha20/)
-                                                {
-                                                    if ($self->{method} eq 'chacha20-ietf')
+                                                    elsif($self->{method} =~ /^chacha20/)
                                                     {
+                                                        if ($self->{method} eq 'chacha20-ietf')
+                                                        {
                                                             #my $ct_len = length($incoming_data)  % 64;
                                                             #my $pad_len = 64 - $ct_len;
                                                             #my $padded_incoming_data  = Net::Shadowsocks::_add_padding($incoming_data,$pad_len); 
@@ -286,25 +282,81 @@ package Net::Shadowsocks::Client;
                                                             #$decrypted_data = Net::Shadowsocks::_remove_padding($padded_decrypted_data,$pad_len);
                                                             #$decrypted_data = $decryptor->chacha20_ietf_xor($incoming_data,$server_iv ,$key);
                                                             $decrypt_counter +=  length($incoming_data) / 64;
-                                                    }
-                                                    else
-                                                    {
+                                                        }
+                                                        else
+                                                        {
                                                          #my $chunk_len = 0;
-                                                         while (length($incoming_data) > 0)
-                                                         {
+                                                            while (length($incoming_data) > 0)
+                                                            {
+                                                                if (length($incoming_data) < 34)
+                                                                {
+                                                                    return;
+                                                                }
+                                                                else
+                                                                {
+                                                                    my $data_len_ct = substr($incoming_data,0,18); 
+                                                                    my $data_len_pt;
+                                                                    eval
+                                                                    {
+                                                                        $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, '', $decrypt_nonce, $decrypt_subkey);
+                                                                    };
+                                                                    if ( $@ ) 
+                                                                    {
+                                                                        AE::log error =>  "data length forged!";
+                                                                        $rhandle->destroy();
+                                                                    } 
+                                                                    else 
+                                                                    {
+                                                                        my $data_len = unpack('n',$data_len_pt); 
+                                                                        #carp "Decrypted data length: $data_len\n";
+                                                                        my $chunk_len = $data_len + 34;
+                                                                    
+                                                                        #$chunk_len_total += $chunk_len;
+                                                                        if (length($incoming_data) < $chunk_len)
+                                                                        {
+                                                                            return;
+                                                                        }
+                                                                        else
+                                                                        {
+                                                                            $decrypt_nonce = $decrypt_nonce->increment();
+                                                                            $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),'',$decrypt_nonce,$decrypt_subkey);
+                                                                            if ( $@ ) 
+                                                                            {
+                                                                                AE::log error =>  "data forged!";
+                                                                                $rhandle->destroy();
+                                                                            }
+                                                                            else
+                                                                            {
+                                                                                $decrypt_nonce = $decrypt_nonce->increment(); 
+                                                                                $incoming_data = substr($incoming_data,$chunk_len);                                                                       
+                                                                                $rhandle->{rbuf} = $incoming_data;
+                                                                                carp "more chunk to process" if (length($incoming_data ) > 0) 
+                                                                            }  
+                                                                        }      
+                                                                    }
+                                                                }
+                                                            }
+                                                        }
+                                                    }
+                                                    elsif($self->{method} =~ /gcm$/)
+                                                    {
+                                                        #my $chunk_len = 0;
+                                                        while (length($incoming_data) > 0)
+                                                        {
                                                             if (length($incoming_data) < 34)
                                                             {
                                                                 return;
                                                             }
                                                             else
                                                             {
-                                                                my $data_len_ct = substr($incoming_data,0,18); 
-                                                                my $data_len_pt;
-                                                                eval
-                                                                {
-                                                                    $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, '', $decrypt_nonce, $decrypt_subkey);
-                                                                };
-                                                                if ( $@ ) 
+                                                                my $data_len_ct = substr($incoming_data,0,2); 
+                                                                $decryptor->reset();
+                                                                $decryptor->iv_add($decrypt_nonce);
+                                                                $decryptor->adata_add('');
+                                                                my $data_len_pt = $decryptor->decrypt_add($data_len_ct);
+                                                                my $data_len_tag = substr($incoming_data,2,16);
+                                                                my $length_result = $decryptor->decrypt_done($data_len_tag);
+                                                                if (!$length_result)
                                                                 {
                                                                     AE::log error =>  "data length forged!";
                                                                     $rhandle->destroy();
@@ -314,8 +366,6 @@ package Net::Shadowsocks::Client;
                                                                     my $data_len = unpack('n',$data_len_pt); 
                                                                     #carp "Decrypted data length: $data_len\n";
                                                                     my $chunk_len = $data_len + 34;
-                                                                    
-                                                                    #$chunk_len_total += $chunk_len;
                                                                     if (length($incoming_data) < $chunk_len)
                                                                     {
                                                                         return;
@@ -323,95 +373,41 @@ package Net::Shadowsocks::Client;
                                                                     else
                                                                     {
                                                                         $decrypt_nonce = $decrypt_nonce->increment();
-                                                                        $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),'',$decrypt_nonce,$decrypt_subkey);
-                                                                         if ( $@ ) 
-                                                                         {
-                                                                             AE::log error =>  "data forged!";
-                                                                             $rhandle->destroy();
-                                                                         }
-                                                                         else
-                                                                         {
-                                                                             $decrypt_nonce = $decrypt_nonce->increment(); 
-                                                                             $incoming_data = substr($incoming_data,$chunk_len);                                                                       
-                                                                             $rhandle->{rbuf} = $incoming_data;
-                                                                             carp "more chunk to process" if (length($incoming_data ) > 0) 
-                                                                         }  
+                                                                        $decryptor->reset();
+                                                                        $decryptor->iv_add($decrypt_nonce);
+                                                                        $decryptor->adata_add('');
+                                                                        $decrypted_data .= $decryptor->decrypt_add(substr($incoming_data,18,$data_len ));
+                                                                        my $data_tag = substr($incoming_data,18 + $data_len,16);
+                                                                        my $data_result = $decryptor->decrypt_done($data_tag);
+                                                                        if ( !$data_result ) 
+                                                                        {
+                                                                            AE::log error =>  "data forged!";
+                                                                            $rhandle->destroy();
+                                                                        }
+                                                                        else
+                                                                        {
+                                                                            $decrypt_nonce = $decrypt_nonce->increment();  
+                                                                            $decryptor->reset();     
+                                                                            $incoming_data = substr($incoming_data,$chunk_len);                                                                       
+                                                                            $rhandle->{rbuf} = $incoming_data;     
+                                                                            carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                      
+                                                                        }  
                                                                     }      
                                                                 }
                                                             }
                                                         }
+                                                    }                                              
+                                                    else
+                                                    {
+                                                        $decrypted_data = $decryptor->decrypt($incoming_data,$key,$server_iv);
                                                     }
-                                                }
-                                                elsif($self->{method} =~ /gcm$/)
-                                                {
-                                                     #my $chunk_len = 0;
-                                                     while (length($incoming_data) > 0)
-                                                     {
-                                                        if (length($incoming_data) < 34)
-                                                        {
-                                                            return;
-                                                        }
-                                                        else
-                                                        {
-                                                            my $data_len_ct = substr($incoming_data,0,2); 
-                                                            $decryptor->reset();
-                                                            $decryptor->iv_add($decrypt_nonce);
-                                                            $decryptor->adata_add('');
-                                                            my $data_len_pt = $decryptor->decrypt_add($data_len_ct);
-                                                            my $data_len_tag = substr($incoming_data,2,16);
-                                                            my $length_result = $decryptor->decrypt_done($data_len_tag);
-                                                            if (!$length_result)
-                                                            {
-                                                                AE::log error =>  "data length forged!";
-                                                                $rhandle->destroy();
-                                                            } 
-                                                            else 
-                                                            {
-                                                                my $data_len = unpack('n',$data_len_pt); 
-                                                                #carp "Decrypted data length: $data_len\n";
-                                                                my $chunk_len = $data_len + 34;
-                                                                if (length($incoming_data) < $chunk_len)
-                                                                {
-                                                                    return;
-                                                                }
-                                                                else
-                                                                {
-                                                                    $decrypt_nonce = $decrypt_nonce->increment();
-                                                                    $decryptor->reset();
-                                                                    $decryptor->iv_add($decrypt_nonce);
-                                                                    $decryptor->adata_add('');
-                                                                    $decrypted_data .= $decryptor->decrypt_add(substr($incoming_data,18,$data_len ));
-                                                                    my $data_tag = substr($incoming_data,18 + $data_len,16);
-                                                                    my $data_result = $decryptor->decrypt_done($data_tag);
-                                                                     if ( !$data_result ) 
-                                                                     {
-                                                                         AE::log error =>  "data forged!";
-                                                                         $rhandle->destroy();
-                                                                     }
-                                                                     else
-                                                                     {
-                                                                         $decrypt_nonce = $decrypt_nonce->increment();  
-                                                                         $decryptor->reset();     
-                                                                         $incoming_data = substr($incoming_data,$chunk_len);                                                                       
-                                                                         $rhandle->{rbuf} = $incoming_data;     
-                                                                          carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                      
-                                                                     }  
-                                                                }      
-                                                            }
-                                                        }
+                                                    $chandle->push_write($decrypted_data);
+                                                    if ($self->{method} !~ /gcm$/ and $self->{method} !~ /poly1305$/)
+                                                    {
+                                                        $rhandle->{rbuf} = '';
                                                     }
-                                                }                                              
-                                                else
-                                                {
-                                                    $decrypted_data = $decryptor->decrypt($incoming_data,$key,$server_iv);
-                                                }
-                                                $chandle->push_write($decrypted_data);
-                                                if ($self->{method} !~ /gcm$/ and $self->{method} !~ /poly1305$/)
-                                                {
-                                                    $rhandle->{rbuf} = '';
-                                                }
+                                                 }
                                              }
-                                         }
                                          );
                                     }                                   
                                 }
@@ -448,13 +444,12 @@ package Net::Shadowsocks::Client;
                                     $plain_data = $addr_to_send . $client_buffer;
                                     $addr_to_send = '';
                                 }
-=cut for comment
-                                if ( $self->{method} =~ /^rc4/  ) 
+
+                                if ( $self->{method} eq 'rc4-md5'  ) 
                                 {
-                                    $encrypted_data = Mcrypt::mcrypt_encrypt($encryptor,$plain_data);                                
+                                    $encrypted_data = $encryptor->RC4($plain_data);                                
                                 }
-=cut
-                                if ( $self->{method} eq "rabbit" or $self->{method} eq "spritz" ) 
+                                elsif ( $self->{method} eq "rabbit" or $self->{method} eq "spritz" ) 
                                 {
                                     $encrypted_data = $encryptor->encrypt($plain_data);                                
                                 }
@@ -564,7 +559,7 @@ Net::Shadowsocks::Client - Shadowsocks protocol client module.
 
 =head1 VERSION
 
-Version 0.9.1
+Version 0.9.2
 
 =head1 SYNOPSIS
 
@@ -572,7 +567,7 @@ Version 0.9.1
     
 =head1 DESCRIPTION
 
-1. A total of 34 encryption methods are supported:
+1. A total of 35 encryption methods are supported:
 
 	aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
 	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
@@ -582,6 +577,7 @@ Version 0.9.1
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
 	rabbit
+	rc4-md5
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
@@ -589,7 +585,7 @@ Version 0.9.1
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 
-      bf-cfb chacha20 salsa20 rc4-md5
+      bf-cfb chacha20 salsa20 
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
  
@@ -619,7 +615,7 @@ Project website https://osdn.net/projects/ssperl/
     password => '49923641',
     server => 'jp.ssip.club',
     server_port => 23333,
-    method => 'rc6',
+    method => 'spritz',
     );
 
     This is all you need to do. Take a look at client.pl under eg directory for a compelete example on how to
