@@ -32,7 +32,7 @@
     use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
     #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.1';
+    our $VERSION = '0.9.2';
     
     $AnyEvent::Log::FILTER->level ("info");
     
@@ -50,7 +50,10 @@
          {
             undef $self->{server};
         }
-
+        unless (defined($self->{method}))
+        {
+            $self->{method} = 'spritz';
+        } 
         my $tcp_server;
         $tcp_server = AnyEvent::Socket::tcp_server
         (
@@ -110,25 +113,17 @@
                                     $client_iv = substr( $client_buffer, 0, length($iv) );
                                     $incoming_data = substr( $client_buffer, length($iv) );
                                     $chandle->{rbuf} = $incoming_data;
-=cut for comment
-                                    if ( !defined($self->{method}) or $self->{method} eq 'rc4-md5' ) 
+
+                                    if ( $self->{method} eq 'rc4-md5' ) 
                                     {
                                         my $md = Digest::MD5->new();
                                         $md->add($key .$client_iv );
                                         #$md->add($client_iv);
                                         my $decrypt_rc4_key = $md->digest();
-                                        Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
+                                        $decryptor = Crypt::RC4::XS->new($decrypt_rc4_key);
                                     }
-                                    elsif($self->{method} eq 'rc4-sha')
-                                    {
-                                        my $sha = Digest::SHA->new();
-                                        $sha->add($key .$client_iv );
-                                        #$md->add($client_iv);
-                                        my $decrypt_rc4_key = substr($sha->digest(),0,16);
-                                        Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
-                                    }
-=cut
-                                    if ( !defined($self->{method}) or $self->{method} eq 'spritz' ) 
+
+                                    elsif ( !defined($self->{method}) or $self->{method} eq 'spritz' ) 
                                     {
                                         $decryptor = Crypt::Spritz::Cipher->new($key,$client_iv);
                                     }
@@ -159,13 +154,12 @@
                              {      
                                  if ($mode == 0)
                                  {
-=cut for comment
-                                     if ($self->{method} =~ /^rc4/ )
+                                     if ($self->{method} eq 'rc4-md5' )
                                      {
-                                          $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
+                                          $decrypted_data = $decryptor->RC4($incoming_data );
                                      }
-=cut
-                                     if ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
+
+                                     elsif ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
                                      {
                                           $decrypted_data = $decryptor->decrypt($incoming_data );
                                      }
@@ -419,13 +413,13 @@
                                                       my $remote_buffer = $rhandle->rbuf();
                                                       my $plain_data = $remote_buffer;
                                                       my $encrypted_data;
-=cut for comment                                                
-                                                      if ( $self->{method} =~ /^rc4/)
+                                               
+                                                      if ( $self->{method} eq 'rc4-md5')
                                                       {
-                                                          $encrypted_data = Mcrypt::mcrypt_encrypt($encryptor, $plain_data );    
+                                                          $encrypted_data = $encryptor->RC4($plain_data );    
                                                       }
-=cut
-                                                      if ( $self->{method} eq "rabbit" or $self->{method} eq "spritz")
+
+                                                      elsif ( $self->{method} eq "rabbit" or $self->{method} eq "spritz")
                                                       {
                                                           $encrypted_data = $encryptor->encrypt($plain_data );    
                                                       }
@@ -503,13 +497,12 @@
                                }
                                else
                                {
-=cut for comment
-                                     if ($self->{method} =~ /^rc4/ )
+
+                                     if ($self->{method} eq 'rc4-md5' )
                                      {
-                                          $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
+                                          $decrypted_data = $decryptor->RC4($incoming_data );
                                      }
-=cut
-                                     if ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
+                                     elsif ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
                                      {
                                           $decrypted_data = $decryptor->decrypt($incoming_data );
                                      }
@@ -665,16 +658,19 @@
             {
                     my $client_socket = shift;
                     setsockopt($client_socket,,SOL_SOCKET, SO_REUSEADDR, 1);
-                    if ( $_osname eq 'linux' ) 
+                    if ( $_osname eq 'linux' and  -e '/proc/sys/net/ipv4/tcp_fastopen') 
                     {
-                        my $_tfo = do 
+                        if (-r '/proc/sys/net/ipv4/tcp_fastopen')
                         {
-                            local ( @ARGV, $/ ) = '/proc/sys/net/ipv4/tcp_fastopen';<>;
-                        };
-                        if ( $_tfo == 2 or $_tfo == 3 ) 
-                        {
-                            setsockopt( $client_socket, IPPROTO_TCP, TCP_FASTOPEN, 1 );
-                            AE::log info => "TCP Fast Open enabled on server.";
+                            my $_tfo = do 
+                            {
+                                local ( @ARGV, $/ ) = '/proc/sys/net/ipv4/tcp_fastopen';<>;
+                            };
+                            if ( $_tfo == 2 or $_tfo == 3 ) 
+                            {
+                                setsockopt( $client_socket, IPPROTO_TCP, TCP_FASTOPEN, 1 );
+                                AE::log info => "TCP Fast Open enabled on server.";
+                            }
                         }
                     }
                     elsif ( $_osname eq 'darwin' ) 
@@ -702,11 +698,11 @@
     
 =head1 NAME
 
-Net::Shadowsocks::Server - Shadowsocks protocol server.
+Net::Shadowsocks::Server - Shadowsocks protocol server module.
 
 =head1 VERSION
 
-Version 0.9.1
+Version 0.9.2
 
 =head1 SYNOPSIS
 
@@ -724,6 +720,7 @@ Version 0.9.1
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
 	rabbit
+	rc4-md5
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
@@ -731,7 +728,7 @@ Version 0.9.1
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 
-      bf-cfb chacha20 salsa20 rc4-md5
+      bf-cfb chacha20 salsa20 
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
  
@@ -759,7 +756,7 @@ Project website https://osdn.net/projects/ssperl/
     password => ' 49923641 ',
     server => ' jp . ssip . club ',
     server_port => 23333,
-    method => 'rc6',
+    method => 'spritz',
     );
 
     This is all you need to do. Take a look at server.pl under eg directory for a compelete example on how to
